# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

# standard imports
import datetime
import os
import platform
import signal
import sys
import time
import json
import logging
import logging.handlers

# third party imports
from darksky import forecast
import pygame
import requests


# globals
UNICODE_DEGREE = u'\xb0'


def exit_gracefully(signum, frame):
    sys.exit(0)


signal.signal(signal.SIGTERM, exit_gracefully)


class Weather:
    """
    Fetches weather reports from Dark Sky for displaying on a screen.
    """

    def __init__(self, config_file):
        with open(config_file, "r") as f:
            self.config = json.load(f)

        self.last_update_check = 0
        self.weather = {}
        self.get_forecast()
        # Initialize logger
        self.log = self.get_logger()

        if platform.system() == 'Darwin':
            pygame.display.init()
            driver = pygame.display.get_driver()
            self.log.debug(f"Using the {driver} driver.")
        else:
            # Based on "Python GUI in Linux frame buffer"
            # http://www.karoltomala.com/blog/?p=679
            disp_no = os.getenv("DISPLAY")
            if disp_no:
                self.log.debug(f"X Display = {disp_no}")

            # Check which frame buffer drivers are available
            # Start with fbcon since directfb hangs with composite output
            drivers = ['x11', 'fbcon', 'directfb', 'svgalib']
            found = False
            for driver in drivers:
                # Make sure that SDL_VIDEODRIVER is set
                if not os.getenv('SDL_VIDEODRIVER'):
                    os.putenv('SDL_VIDEODRIVER', driver)
                try:
                    pygame.display.init()
                except pygame.error:
                    self.log.debug("Driver: {driver} failed.")
                    continue
                found = True
                break

            if not found:
                self.log.exception("No suitable video driver found!")

        size = (pygame.display.Info().current_w,
                pygame.display.Info().current_h)
        self.sizing(size)

        # Clear the screen to start
        self.screen.fill((0, 0, 0))
        # Initialise font support
        pygame.font.init()
        # Render the screen
        pygame.mouse.set_visible(0)
        pygame.display.update()

        self.subwindow_text_height = 0.055
        self.time_date_text_height = 0.115
        self.time_date_small_text_height = 0.075
        self.time_date_y_position = 8
        self.time_date_small_y_position = 18

    def __del__(self):
        "Destructor to make sure pygame shuts down, etc."

    def sizing(self, size):
        """
        Set various asplect of the app related to the screen size of
        the display and/or window.
        """

        self.log.debug(f"Framebuffer Size: {size[0]} x {size[1]}")

        if self.config["fullscreen"]:
            self.screen = pygame.display.set_mode(size, pygame.FULLSCREEN)
            self.xmax = pygame.display.Info().current_w - 35
            self.ymax = pygame.display.Info().current_h - 5
        else:
            self.screen = pygame.display.set_mode(size, pygame.RESIZABLE)
            pygame.display.set_caption('PiWeatherRock')
            self.xmax = pygame.display.get_surface().get_width() - 35
            self.ymax = pygame.display.get_surface().get_height() - 5

        if self.xmax <= 1024:
            self.icon_size = '64'
        else:
            self.icon_size = '256'

    def get_logger(self):
        """
        Create a logger to be used for logging messages to a file. The
        verbosity of the logs is determined by the 'log_level' setting in
        the config file.
        """
        lvl_str = f"logging.{self.config['log_level']}"
        log = logging.getLogger()
        log.setLevel(eval(lvl_str))
        formatter = logging.Formatter(
            "%(asctime)s %(levelname)-8s %(message)s",
            datefmt='%Y-%m-%d %H:%M:%S')
        handler = logging.handlers.RotatingFileHandler(
                ".log", maxBytes=500000, backupCount=3)
        if (log.hasHandlers()):
            log.handlers.clear()
        handler.setFormatter(formatter)
        log.addHandler(handler)

        return log

    def get_forecast(self):
        """
        Gets updated information if the 'update_freq' amount of time has
        passed since last querying the api.
        """
        if (time.time() - self.last_update_check) > self.config["update_freq"]:
            self.last_update_check = time.time()
            try:
                self.weather = forecast(
                    self.config["ds_api_key"],
                    self.config["lat"],
                    self.config["lon"],
                    exclude='minutely',
                    units=self.config["units"],
                    lang=self.config["lang"])

                sunset_today = datetime.datetime.fromtimestamp(
                    self.weather.daily[0].sunsetTime)
                if datetime.datetime.now() < sunset_today:
                    index = 0
                    sr_suffix = 'today'
                    ss_suffix = 'tonight'
                else:
                    index = 1
                    sr_suffix = 'tomorrow'
                    ss_suffix = 'tomorrow'

                self.sunrise = self.weather.daily[index].sunriseTime
                self.sunrise_string = datetime.datetime.fromtimestamp(
                    self.sunrise).strftime("%I:%M %p {}").format(sr_suffix)
                self.sunset = self.weather.daily[index].sunsetTime
                self.sunset_string = datetime.datetime.fromtimestamp(
                    self.sunset).strftime("%I:%M %p {}").format(ss_suffix)

            except requests.exceptions.RequestException as e:
                self.log.exception(f"Request exception: {e}")
                return False
            except AttributeError as e:
                self.log.exception(f"Attribute error: {e}")
                return False
        return True

    def screen_cap(self):
        """
        Save a jpg image of the screen
        """
        pygame.image.save(self.screen, "screenshot.jpeg")
        self.log.info("Screen capture complete.")



# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

import json
import pygame
import sys
import time

# pylint is mad about thise locals regardless of how they are used. with
# that being the case, I decided to have the lint error here instead of
# every place they get used. PR's welcome to make pylint happy about this
# and pygame.quit()
from pygame.locals import QUIT, VIDEORESIZE, KEYDOWN, K_KP_ENTER, K_q, K_d, K_h, K_i, K_s

# local imports
from piweatherrock.weather import Weather
from piweatherrock.plugin_weather_daily import PluginWeatherDaily
from piweatherrock.plugin_weather_hourly import PluginWeatherHourly
from piweatherrock.plugin_info import PluginInfo


class Runner:

    def __init__(self):
        self.current_screen = None
        self.d_count = 1
        self.h_count = 0
        self.running = False
        self.seconds = 0
        self.non_weather_timeout = 0
        self.periodic_info_activation = 0
        self.config = None
        self.my_weather_rock = None
        self.daily = None
        self.hourly = None
        self.info = None

    def main(self, config_file):
        with open(config_file, "r") as f:
            self.config = json.load(f)

        # Create an instance of the main application class
        self.my_weather_rock = Weather(config_file)

        # Create an instance of each plugin that will be used
        self.daily = PluginWeatherDaily(self.my_weather_rock)
        self.hourly = PluginWeatherHourly(self.my_weather_rock)
        self.info = PluginInfo(self.my_weather_rock)

        # Default to weather mode. Showing daily weather first.
        self.current_screen = 'd'

        self.d_count = 1
        self.h_count = 0

        # Stay running while True
        self.running = True

        # Seconds Placeholder to pace display
        self.seconds = 0

        # Display timeout to automatically switch back to weather display.
        self.non_weather_timeout = 0

        # Switch to info periodically to prevent screen burn.
        self.periodic_info_activation = 0

        # Loads data from darksky.net
        if not self.my_weather_rock.get_forecast():
            self.my_weather_rock.log.exception(
                "Error: no data from darksky.net.")
            self.running = False

        ##################################################################
        #                        Main progam loop                        #
        ##################################################################
        while self.running:
            # Look for and process keyboard events to change modes.
            self.process_pygame_events()
            self.screen_switcher()

            # Loop timer.
            pygame.time.wait(100)

        # When the main program loop is exited, exit the application
        pygame.quit()

    def process_pygame_events(self):
        """
        pygame events are how we learn about a window being closed or
        resized or a key being pressed. This function looks for the events
        we care about and reacts when needed.
        """

        for event in pygame.event.get():
            if event.type == QUIT:
                self.running = False
            elif event.type == VIDEORESIZE:
                self.my_weather_rock.sizing(event.size)
            elif event.type == KEYDOWN:

                # On 'q' or keypad enter key, quit the program.
                if ((event.key == K_KP_ENTER) or (event.key == K_q)):
                    self.running = False

                # On 'd' key, set mode to 'daily weather'.
                elif event.key == K_d:
                    self.current_screen = 'd'
                    self.d_count = 1
                    self.h_count = 0
                    self.non_weather_timeout = 0
                    self.periodic_info_activation = 0

                # on 'h' key, set mode to 'hourly weather'
                elif event.key == K_h:
                    self.current_screen = 'h'
                    self.d_count = 0
                    self.h_count = 1
                    self.non_weather_timeout = 0
                    self.periodic_info_activation = 0

                # On 'i' key, set mode to 'info'.
                elif event.key == K_i:
                    self.current_screen = 'i'
                    self.d_count = 0
                    self.h_count = 0
                    self.non_weather_timeout = 0
                    self.periodic_info_activation = 0

                # On 's' key, save a screen shot.
                elif event.key == K_s:
                    self.my_weather_rock.screen_cap()

    def screen_switcher(self):
        """
        This function takes care of cycling through the different screens
        on a regular basis.
        """

        # Automatically switch back to weather display after a couple minutes
        if self.current_screen not in ('d', 'h'):
            self.periodic_info_activation = 0
            self.non_weather_timeout += 1
            self.d_count = 0
            self.h_count = 0

            # Default in config.json.sample: pause for 5 minutes on info screen
            if self.non_weather_timeout > (self.config["info_pause"] * 10):
                self.current_screen = 'd'
                self.d_count = 1
                self.my_weather_rock.log.info("Switching to weather mode")
        else:
            self.non_weather_timeout = 0
            self.periodic_info_activation += 1

            # Default is to flip between 2 weather screens
            # for 15 minutes before showing info screen.
            if self.periodic_info_activation > (self.config["info_delay"] * 10):
                self.current_screen = 'i'
                self.my_weather_rock.log.info("Switching to info mode")
            elif (self.periodic_info_activation % (
                    ((self.config["plugins"]["daily"]["pause"] * self.d_count)
                        + (self.config["plugins"]["hourly"]["pause"] * self.h_count))
                    * 10)) == 0:
                if self.current_screen == 'd':
                    self.my_weather_rock.log.info("Switching to HOURLY")
                    self.current_screen = 'h'
                    self.h_count += 1
                else:
                    self.my_weather_rock.log.info("Switching to DAILY")
                    self.current_screen = 'd'
                    self.d_count += 1

        # Daily Weather Display Mode
        if self.current_screen == 'd':
            # Update / Refresh the display after each second.
            if self.seconds != time.localtime().tm_sec:
                self.seconds = time.localtime().tm_sec
                self.daily.disp_daily(self.my_weather_rock)

            # Once the screen is updated, we have a full second to get the
            # weather. Once per minute, check to see if its time to get a
            # new set of data from the API.
            if self.seconds == 0:
                self.check_forecast()

        # Hourly Weather Display Mode
        elif self.current_screen == 'h':
            # Update / Refresh the display after each second.
            if self.seconds != time.localtime().tm_sec:
                self.seconds = time.localtime().tm_sec
                self.hourly.disp_hourly(self.my_weather_rock)

            # Once the screen is updated, we have a full second to get the
            # weather. Once per minute, check to see if its time to get a
            # new set of data from the API.
            if self.seconds == 0:
                self.check_forecast()

        # Info Screen Display Mode
        elif self.current_screen == 'i':
            # Pace the screen updates to once per second.
            if self.seconds != time.localtime().tm_sec:
                self.seconds = time.localtime().tm_sec

                # Disaplay information about the application along with the
                # time of sunrise and sunset.
                self.info.disp_info(self.my_weather_rock)

    def check_forecast(self):
        try:
            self.my_weather_rock.get_forecast()
        # includes simplejson.decoder.JSONDecodeError
        except ValueError:
            self.my_weather_rock.log.exception(
                f"Decoding JSON has failed: {sys.exc_info()[0]}")
        except BaseException:
            self.my_weather_rock.log.exception(
                f"Unexpected error: {sys.exc_info()[0]}")



# Pulled from https://github.com/grantwinney/52-Weeks-of-Pi/blob/master/GPIOmock.py
#
# Created by the comments in source/py_gpio.c, in the RPi.GPIO package
# available here -> https://pypi.python.org/pypi/RPi.GPIO
# or here -> https://github.com/Tieske/rpi-gpio/blob/master/source/py_gpio.c

HIGH = 1
LOW = 0

BCM = 'BroadCom Numbering'
BOARD = 'Board Numbering'

IN = '"In" Direction for Pin'
OUT = '"Out" Direction for Pin'

PUD_OFF = "Pull Up/Down Resitor Disabled"
PUD_UP = "Pull Up Resistor"
PUD_DOWN = "Pull Down Resistor"

RISING = "Rising Edge"
FALLING = "Falling Edge"
BOTH = "Rising and Falling Edges"


# Clean up by resetting all GPIO channels that have been used by this program to INPUT
#   with no pullup/pulldown and no event detection
# [channel] - individual channel or list/tuple of channels to clean up.
# Default - clean every channel that has been used."},
def cleanup(channel=None):
    print("Reset all GPIO channels (or channel {}) to INPUT with no pullup/pulldown and no event detection."
          .format(channel))


# Set up a GPIO channel or list of channels with a direction and (optional) pull/up down control
# channel        - either board pin number or BCM number depending on which mode is set.
# direction      - IN or OUT
# [pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN
# [initial]      - Initial value for an output channel
def setup(channels, direction, pull_up_down=PUD_OFF, initial=None):
    print("Setup channel(s) {} for direction {}, with pull_up_down {} and set to initial value {}"
          .format(channels, direction, pull_up_down, initial))


# Output to a GPIO channel or list of channels
# channel - either board pin number or BCM number depending on which mode is set.
# value   - 0/1 or False/True or LOW/HIGH
def output(channels, value):
    print("Output a {} value to channel(s) {}".format(value, channels))


# Input from a GPIO channel. Returns HIGH=1=True or LOW=0=False
# channel - either board pin number or BCM number depending on which mode is set.
def input(channel):
    print("Returns the current input value (HIGH or LOW) of channel {}".format(channel))


# Set up numbering mode to use for channels.
# BOARD - Use Raspberry Pi board numbers
# BCM   - Use Broadcom GPIO 00..nn numbers
def setmode(mode):
    print("Numbering style set to {} style".format(mode))


# Get numbering mode used for channel numbers.
# Returns BOARD, BCM or None
def getmode():
    print("Returns the numbering mode of BOARD, BCM or None")


# Add a callback for an event already defined using add_event_detect()
# channel      - either board pin number or BCM number depending on which mode is set.
# callback     - a callback function
def add_event_callback(gpio, callback):
    print("Also call {} when edge detected on channel {}".format(callback, gpio))


# Enable edge detection events for a particular GPIO channel.
# channel      - either board pin number or BCM number depending on which mode is set.
# edge         - RISING, FALLING or BOTH
# [callback]   - A callback function for the event (optional)
# [bouncetime] - Switch bounce timeout in ms for callback
def add_event_detect(gpio, edge, callback=None, bouncetime=None):
    print("Added event detection to PIN {} on edge {}. Call {} with minimal bounce time of {}."
          .format(gpio, edge, callback, bouncetime))


# Remove edge detection for a particular GPIO channel
# channel - either board pin number or BCM number depending on which mode is set.
def remove_event_detect(gpio):
    print("Edge detection removed for GPIO channel {}".format(gpio))


# Returns True if an edge has occurred on a given GPIO.
# You need to enable edge detection using add_event_detect() first.
# channel - either board pin number or BCM number depending on which mode is set.
def event_detected(channel):
    print("An event was detected on channel {}".format(channel))


# Wait for an edge.  Returns the channel number or None on timeout.
# channel      - either board pin number or BCM number depending on which mode is set.
# edge         - RISING, FALLING or BOTH
# [bouncetime] - time allowed between calls to allow for switch-bounce
# [timeout]    - timeout in ms
def wait_for_edge(channel, edge, bouncetime=None, timeout=None):
    print("Waiting for edge {} on channel {}, with a minimal time between calls of {} and a timeout of {}."
          .format(channel, edge, bouncetime, timeout))


# Return the current GPIO function (IN, OUT, PWM, SERIAL, I2C, SPI)
# channel - either board pin number or BCM number depending on which mode is set.
def gpio_function(channel):
    print("Returns the current GPIO function for channel {}".format(channel))


# Enable or disable warning messages
def setwarnings(state):
    print("Set warnings to {}".format(state))


def PWM(channel, frequency):
    return PwmMock(channel, frequency)


# Prints information about your Raspberry Pi
RPI_INFO = "{'P1_REVISION': -1, 'RAM': '-1M', 'REVISION': '-1', 'TYPE': 'Pi Model', 'PROCESSOR': 'Some Processor', 'MANUFACTURER': 'Some Mfr'}"

# Prints the version of RPi.GPIO
VERSION = "0.0.0"


class PwmMock:
    p_channel = 0

    def __init__(self, channel, frequency):
        global p_channel
        p_channel = channel
        print("New instance of class created: {}. Enables pulse-width modulation on channel {} at frequency {}".format(self, channel, frequency))

    def start(self, frequency):
        print("Start pulse-width modulation at {} for channel {}".format(frequency, p_channel))

    def ChangeDutyCycle(self, frequency):
        print("Set pulse-width modulation to {} for channel {}".format(frequency, p_channel))

    def stop(self):
        print("Stop pulse-width modulation on channel {}".format(p_channel))




# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

import datetime
import pygame
import time

from os import path

UNICODE_DEGREE = u'\xb0'


class PluginWeatherCommon:
    """
    This plugin is resposible for displaying the information on the top
    half of the screen when either the daily or hourly forecast is being
    displayed on the lower half. Both 'plugin_weather_daily' and
    'plugin_weather_hourly' call display_weather_top() to do this.

    This plugin also provides the display_subwindow() function that is used
    by 'plugin_weather_daily' and 'plugin_weather_hourly'.
    """

    def __init__(self, weather_rock):
        self.screen = None
        self.weather = None
        self.config = None
        self.take_umbrella = None
        self.xmax = None
        self.ymax = None
        self.time_date_small_text_height = None
        self.time_date_text_height = None
        self.time_date_y_position = None
        self.time_date_small_y_position = None
        self.subwindow_text_height = None
        self.icon_size = None

        self.get_rock_values(weather_rock)

    def get_rock_values(self, weather_rock):
        self.screen = weather_rock.screen
        self.weather = weather_rock.weather
        self.config = weather_rock.config
        self.take_umbrella = self.umbrella_needed()
        self.xmax = weather_rock.xmax
        self.ymax = weather_rock.ymax
        self.time_date_small_text_height = weather_rock.time_date_small_text_height
        self.time_date_text_height = weather_rock.time_date_text_height
        self.time_date_y_position = weather_rock.time_date_y_position
        self.time_date_small_y_position = weather_rock.time_date_small_y_position
        self.subwindow_text_height = weather_rock.subwindow_text_height
        self.icon_size = weather_rock.icon_size

    def disp_weather_top(self, weather_rock):
        self.get_rock_values(weather_rock)

        # Fill the screen with black
        self.screen.fill((0, 0, 0))
        xmin = 10
        lines = 5
        line_color = (255, 255, 255)
        text_color = (255, 255, 255)
        font_name = "freesans"

        self.draw_screen_border(line_color, xmin, lines)
        self.disp_time_date(font_name, text_color)
        self.disp_current_temp(font_name, text_color)
        self.disp_summary()
        self.display_conditions_line(
            'Feels Like:', int(round(self.weather.apparentTemperature)),
            True)

        try:
            wind_bearing = self.weather.windBearing
            wind_direction = self.deg_to_compass(wind_bearing) + ' @ '
        except AttributeError:
            wind_direction = ''
        wind_txt = wind_direction + str(
            int(round(self.weather.windSpeed))) + \
            ' ' + self.get_windspeed_abbreviation(self.config["units"])
        self.display_conditions_line(
            'Wind:', wind_txt, False, 1)

        self.display_conditions_line(
            'Humidity:', str(int(round((self.weather.humidity * 100)))) + '%',
            False, 2)

        # Skipping multiplier 3 (line 4)

        if self.take_umbrella:
            umbrella_txt = 'Grab your umbrella!'
        else:
            umbrella_txt = 'No umbrella needed today.'
        self.disp_umbrella_info(umbrella_txt)

    def draw_screen_border(self, line_color, xmin, lines):
        # Draw Screen Border
        # Top
        pygame.draw.line(self.screen, line_color, (xmin, 0), (self.xmax, 0),
                         lines)
        # Left
        pygame.draw.line(self.screen, line_color, (xmin, 0),
                         (xmin, self.ymax), lines)
        # Bottom
        pygame.draw.line(self.screen, line_color, (xmin, self.ymax),
                         (self.xmax, self.ymax), lines)
        # Right
        pygame.draw.line(self.screen, line_color, (self.xmax, 0),
                         (self.xmax, self.ymax + 2), lines)
        # Bottom of top box
        pygame.draw.line(self.screen, line_color, (xmin, self.ymax * 0.15),
                         (self.xmax, self.ymax * 0.15), lines)
        # Bottom of middle box
        pygame.draw.line(self.screen, line_color, (xmin, self.ymax * 0.5),
                         (self.xmax, self.ymax * 0.5), lines)
        # Bottom row, left vertical
        pygame.draw.line(self.screen, line_color, (self.xmax * 0.25,
                                                   self.ymax * 0.5),
                         (self.xmax * 0.25, self.ymax), lines)
        # Bottom row, center vertical
        pygame.draw.line(self.screen, line_color, (self.xmax * 0.5,
                                                   self.ymax * 0.15),
                         (self.xmax * 0.5, self.ymax), lines)
        # Bottom row, right vertical
        pygame.draw.line(self.screen, line_color, (self.xmax * 0.75,
                                                   self.ymax * 0.5),
                         (self.xmax * 0.75, self.ymax), lines)

    def disp_time_date(self, font_name, text_color):
        # Time & Date
        time_date_font = pygame.font.SysFont(
            font_name, int(self.ymax * self.time_date_text_height), bold=1)
        # Small Font for Seconds
        small_font = pygame.font.SysFont(
            font_name,
            int(self.ymax * self.time_date_small_text_height), bold=1)

        time_string = time.strftime("%a, %b %d   %I:%M", time.localtime())
        am_pm_string = time.strftime(" %p", time.localtime())

        rendered_time_string = time_date_font.render(time_string, True,
                                                     text_color)
        (rendered_time_x, rendered_time_y) = rendered_time_string.get_size()
        rendered_am_pm_string = small_font.render(am_pm_string, True,
                                                  text_color)
        (rendered_am_pm_x, rendered_am_pm_y) = rendered_am_pm_string.get_size()

        full_time_string_x_position = self.xmax / 2 - (rendered_time_x +
                                                       rendered_am_pm_x) / 2
        self.screen.blit(rendered_time_string, (full_time_string_x_position,
                                                self.time_date_y_position))
        self.screen.blit(rendered_am_pm_string,
                         (full_time_string_x_position + rendered_time_x + 3,
                          self.time_date_small_y_position))

    def disp_current_temp(self, font_name, text_color):
        # Outside Temp
        outside_temp_font = pygame.font.SysFont(
            font_name, int(self.ymax * (0.5 - 0.15) * 0.6), bold=1)
        txt = outside_temp_font.render(
            str(int(round(self.weather.temperature))), True, text_color)
        (txt_x, txt_y) = txt.get_size()
        degree_font = pygame.font.SysFont(
            font_name, int(self.ymax * (0.5 - 0.15) * 0.3), bold=1)
        degree_txt = degree_font.render(UNICODE_DEGREE, True, text_color)
        (rendered_am_pm_x, rendered_am_pm_y) = degree_txt.get_size()
        degree_letter = outside_temp_font.render(
            self.get_temperature_letter(self.config["units"]),
            True, text_color)
        (degree_letter_x, degree_letter_y) = degree_letter.get_size()
        # Position text
        x = self.xmax * 0.27 - (txt_x * 1.02 + rendered_am_pm_x +
                                degree_letter_x) / 2
        self.screen.blit(txt, (x, self.ymax * 0.20))
        x = x + (txt_x * 1.02)
        self.screen.blit(degree_txt, (x, self.ymax * 0.2))
        x = x + (rendered_am_pm_x * 1.02)
        self.screen.blit(degree_letter, (x, self.ymax * 0.2))

    def disp_summary(self):
        y_start_position = 0.444
        conditions_text_height = 0.04
        text_color = (255, 255, 255)
        font_name = "freesans"

        conditions_font = pygame.font.SysFont(
            font_name, int(self.ymax * conditions_text_height), bold=1)
        txt = conditions_font.render(self.weather.summary, True, text_color)
        txt_x = txt.get_size()[0]
        x = self.xmax * 0.27 - (txt_x * 1.02) / 2
        self.screen.blit(txt, (x, self.ymax * y_start_position))

    def display_conditions_line(self, label, cond, is_temp, multiplier=None):
        y_start_position = 0.17
        line_spacing_gap = 0.065
        conditions_text_height = 0.05
        degree_symbol_height = 0.03
        degree_symbol_y_offset = 0.001
        x_start_position = 0.52
        second_column_x_start_position = 0.69
        text_color = (255, 255, 255)
        font_name = "freesans"

        if multiplier is None:
            y_start = y_start_position
        else:
            y_start = (y_start_position + line_spacing_gap * multiplier)

        conditions_font = pygame.font.SysFont(
            font_name, int(self.ymax * conditions_text_height), bold=1)

        txt = conditions_font.render(str(label), True, text_color)

        self.screen.blit(
            txt, (self.xmax * x_start_position, self.ymax * y_start))

        txt = conditions_font.render(str(cond), True, text_color)
        self.screen.blit(txt, (self.xmax * second_column_x_start_position,
                               self.ymax * y_start))

        if is_temp:
            txt_x = txt.get_size()[0]
            degree_font = pygame.font.SysFont(
                font_name, int(self.ymax * degree_symbol_height), bold=1)
            degree_txt = degree_font.render(UNICODE_DEGREE, True, text_color)
            self.screen.blit(degree_txt, (
                self.xmax * second_column_x_start_position + txt_x * 1.01,
                self.ymax * (y_start + degree_symbol_y_offset)))
            degree_letter = conditions_font.render(
                self.get_temperature_letter(self.config["units"]),
                True, text_color)
            degree_letter_x = degree_letter.get_size()[0]
            self.screen.blit(degree_letter, (
                self.xmax * second_column_x_start_position +
                txt_x + degree_letter_x * 1.01,
                self.ymax * (y_start + degree_symbol_y_offset)))

    def deg_to_compass(self, degrees):
        """
        Convert numerical direction into the letters you'd see on a compas
        such as 'N' for north or 'SE' for south east.
        """
        val = int((degrees/22.5)+.5)
        dirs = ["N", "NNE", "NE", "ENE",
                "E", "ESE", "SE", "SSE",
                "S", "SSW", "SW", "WSW",
                "W", "WNW", "NW", "NNW"]
        return dirs[(val % 16)]

    def get_windspeed_abbreviation(self, unit):
        """
        Determines the abbreviation to use for wind speed based on the unit
        a user has chosen (ca, uk2, us, si).
        """
        return self.get_abbreviation(self.units_decoder(unit)['windSpeed'])

    def umbrella_needed(self):
        # start with saying we don't need an umbrella
        take_umbrella = False
        icon_now = self.weather.icon
        icon_today = self.weather.daily[0].icon
        if icon_now == 'rain' or icon_today == 'rain':
            take_umbrella = True
        else:
            # determine if an umbrella is needed during daylight hours
            curr_date = datetime.datetime.today().date()
            for hour in self.weather.hourly:
                hr = datetime.datetime.fromtimestamp(hour.time)
                sr = datetime.datetime.fromtimestamp(
                    self.weather.daily[0].sunriseTime)
                ss = datetime.datetime.fromtimestamp(
                    self.weather.daily[0].sunsetTime)
                rain_chance = hour.precipProbability
                is_today = hr.date() == curr_date
                is_daylight_hr = hr >= sr and hr <= ss
                if is_today and is_daylight_hr and rain_chance >= .25:
                    take_umbrella = True
                    break

        return take_umbrella

    def disp_umbrella_info(self, umbrella_txt):
        x_start_position = 0.52
        y_start_position = 0.444
        conditions_text_height = 0.04
        text_color = (255, 255, 255)
        font_name = "freesans"

        conditions_font = pygame.font.SysFont(
            font_name, int(self.ymax * conditions_text_height), bold=1)
        txt = conditions_font.render(umbrella_txt, True, text_color)
        self.screen.blit(txt, (
            self.xmax * x_start_position,
            self.ymax*y_start_position))

    def get_temperature_letter(self, unit):
        """
        Determines the single letter that represents temperature based on
        unit a user has chosen. ex: 'F' to represent 'Degrees Fahrenheit'
        """
        return self.units_decoder(unit)['temperature'].split(' ')[-1][0].upper()

    def get_abbreviation(self, phrase):
        """
        Create an abbreviation from a phrase by combining the first letter
        of each word in lower case.
        """
        abbreviation = ''.join(item[0].lower() for item in phrase.split())
        return abbreviation

    def units_decoder(self, units):
        """
        https://darksky.net/dev/docs has lists out what each
        unit is. The method below is just a codified version
        of what is on that page.
        """
        si_dict = {
            'nearestStormDistance': 'Kilometers',
            'precipIntensity': 'Millimeters per hour',
            'precipIntensityMax': 'Millimeters per hour',
            'precipAccumulation': 'Centimeters',
            'temperature': 'Degrees Celsius',
            'temperatureMin': 'Degrees Celsius',
            'temperatureMax': 'Degrees Celsius',
            'apparentTemperature': 'Degrees Celsius',
            'dewPoint': 'Degrees Celsius',
            'windSpeed': 'Meters per second',
            'windGust': 'Meters per second',
            'pressure': 'Hectopascals',
            'visibility': 'Kilometers',
        }
        ca_dict = si_dict.copy()
        ca_dict['windSpeed'] = 'Kilometers per hour'
        ca_dict['windGust'] = 'Kilometers per hour'
        uk2_dict = si_dict.copy()
        uk2_dict['nearestStormDistance'] = 'Miles'
        uk2_dict['visibility'] = 'Miles'
        uk2_dict['windSpeed'] = 'Miles per hour'
        uk2_dict['windGust'] = 'Miles per hour'
        us_dict = {
            'nearestStormDistance': 'Miles',
            'precipIntensity': 'Inches per hour',
            'precipIntensityMax': 'Inches per hour',
            'precipAccumulation': 'Inches',
            'temperature': 'Degrees Fahrenheit',
            'temperatureMin': 'Degrees Fahrenheit',
            'temperatureMax': 'Degrees Fahrenheit',
            'apparentTemperature': 'Degrees Fahrenheit',
            'dewPoint': 'Degrees Fahrenheit',
            'windSpeed': 'Miles per hour',
            'windGust': 'Miles per hour',
            'pressure': 'Millibars',
            'visibility': 'Miles',
        }
        switcher = {
            'ca': ca_dict,
            'uk2': uk2_dict,
            'us': us_dict,
            'si': si_dict,
        }
        return switcher.get(units, "Invalid unit name")

    #######################################################################
    #    Everything above here is used exclusively by disp_weather_top    #
    #######################################################################

    def display_subwindow(self, data, day, c_times):
        subwindow_centers = 0.125
        subwindows_y_start_position = 0.530
        line_spacing_gap = 0.065
        rain_percent_line_offset = 5.95
        rain_present_text_height = 0.060
        text_color = (255, 255, 255)
        font_name = "freesans"

        forecast_font = pygame.font.SysFont(
            font_name, int(self.ymax * self.subwindow_text_height), bold=1)
        rpfont = pygame.font.SysFont(
            font_name, int(self.ymax * rain_present_text_height), bold=1)

        txt = forecast_font.render(day, True, text_color)
        (txt_x, txt_y) = txt.get_size()
        self.screen.blit(txt, (self.xmax *
                               (subwindow_centers * c_times) - txt_x / 2,
                               self.ymax * (subwindows_y_start_position +
                                            line_spacing_gap * 0)))
        if hasattr(data, 'temperatureLow'):
            txt = forecast_font.render(
                str(int(round(data.temperatureLow))) +
                UNICODE_DEGREE +
                ' / ' +
                str(int(round(data.temperatureHigh))) +
                UNICODE_DEGREE + self.get_temperature_letter(
                    self.config["units"]),
                True, text_color)
        else:
            txt = forecast_font.render(
                str(int(round(data.temperature))) +
                UNICODE_DEGREE + self.get_temperature_letter(
                    self.config["units"]),
                True, text_color)
        (txt_x, txt_y) = txt.get_size()
        self.screen.blit(txt, (self.xmax *
                               (subwindow_centers * c_times) - txt_x / 2,
                               self.ymax * (subwindows_y_start_position +
                                            line_spacing_gap * 5)))
        rptxt = rpfont.render(
            str(int(round(data.precipProbability * 100))) + '%',
            True, text_color)
        (txt_x, txt_y) = rptxt.get_size()
        self.screen.blit(rptxt, (self.xmax *
                                 (subwindow_centers * c_times) - txt_x / 2,
                                 self.ymax * (subwindows_y_start_position +
                                              line_spacing_gap *
                                              rain_percent_line_offset)))
        icon = pygame.image.load(
            self.icon_mapping(data.icon, self.icon_size)).convert_alpha()
        (icon_size_x, icon_size_y) = icon.get_size()
        if icon_size_y < 90:
            icon_y_offset = (90 - icon_size_y) / 2
        else:
            icon_y_offset = self.config["icon_offset"]

        self.screen.blit(icon, (self.xmax *
                                (subwindow_centers * c_times) -
                                icon_size_x / 2,
                                self.ymax *
                                (subwindows_y_start_position +
                                 line_spacing_gap
                                 * 1.2) + icon_y_offset))

    def icon_mapping(self, icon, size):
        """
        https://darksky.net/dev/docs has this to say about icons:
        icon optional
        A machine-readable text summary of this data point, suitable for
        selecting an icon for display. If defined, this property will have one
        of the following values: clear-day, clear-night, rain, snow, sleet,
        wind, fog, cloudy, partly-cloudy-day, or partly-cloudy-night.
        (Developers should ensure that a sensible default is defined, as
        additional values, such as hail, thunderstorm, or tornado, may be
        defined in the future.)

        Based on that, this method will map the Dark Sky icon name to the name
        of an icon in this project.
        """
        if icon == 'clear-day':
            icon_path = 'icons/{}/clear.png'.format(size)
        elif icon == 'clear-night':
            icon_path = 'icons/{}/nt_clear.png'.format(size)
        elif icon == 'rain':
            icon_path = 'icons/{}/rain.png'.format(size)
        elif icon == 'snow':
            icon_path = 'icons/{}/snow.png'.format(size)
        elif icon == 'sleet':
            icon_path = 'icons/{}/sleet.png'.format(size)
        elif icon == 'wind':
            icon_path = 'icons/alt_icons/{}/wind.png'.format(size)
        elif icon == 'fog':
            icon_path = 'icons/{}/fog.png'.format(size)
        elif icon == 'cloudy':
            icon_path = 'icons/{}/cloudy.png'.format(size)
        elif icon == 'partly-cloudy-day':
            icon_path = 'icons/{}/partlycloudy.png'.format(size)
        elif icon == 'partly-cloudy-night':
            icon_path = 'icons/{}/nt_partlycloudy.png'.format(size)
        else:
            icon_path = 'icons/{}/unknown.png'.format(size)

        return path.join(path.dirname(__file__), icon_path)



# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

import datetime
import pygame

from piweatherrock.plugin_weather_common import PluginWeatherCommon


class PluginWeatherDaily:
    """
    This plugin is resposible for displaying the screen with the daily
    forecast.
    """

    def __init__(self, weather_rock):
        self.screen = None
        self.weather = None
        self.weather_common = None

    def get_rock_values(self, weather_rock):
        self.screen = weather_rock.screen
        self.weather = weather_rock.weather
        self.weather_common = PluginWeatherCommon(weather_rock)

    def disp_daily(self, weather_rock):
        self.get_rock_values(weather_rock)

        self.weather_common.disp_weather_top(weather_rock)

        # Today
        today = self.weather.daily[0]
        today_string = "Today"
        multiplier = 1
        self.weather_common.display_subwindow(today, today_string, multiplier)

        # counts from 0 to 2
        for future_day in range(3):
            this_day = self.weather.daily[future_day + 1]
            this_day_no = datetime.datetime.fromtimestamp(this_day.time)
            this_day_string = this_day_no.strftime("%A")
            multiplier += 2
            self.weather_common.display_subwindow(
                this_day, this_day_string, multiplier)

        # Update the display
        pygame.display.update()



# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

import datetime
import pygame

from piweatherrock.plugin_weather_common import PluginWeatherCommon


class PluginWeatherHourly:
    """
    This plugin is resposible for displaying the screen with the hourly
    forecast.
    """

    def __init__(self, weather_rock):
        self.screen = None
        self.weather = None
        self.weather_common = None

    def get_rock_values(self, weather_rock):
        self.screen = weather_rock.screen
        self.weather = weather_rock.weather
        self.weather_common = PluginWeatherCommon(weather_rock)

    def disp_hourly(self, weather_rock):
        self.get_rock_values(weather_rock)

        self.weather_common.disp_weather_top(weather_rock)

        # Current hour
        this_hour = self.weather.hourly[0]
        this_hour_24_int = int(datetime.datetime.fromtimestamp(
            this_hour.time).strftime("%H"))
        if this_hour_24_int <= 11:
            ampm = 'a.m.'
        else:
            ampm = 'p.m.'
        this_hour_12_int = int(datetime.datetime.fromtimestamp(
            this_hour.time).strftime("%I"))
        this_hour_string = "{} {}".format(str(this_hour_12_int), ampm)
        multiplier = 1
        self.weather_common.display_subwindow(
            this_hour, this_hour_string, multiplier)

        # counts from 0 to 2
        for future_hour in range(3):
            this_hour = self.weather.hourly[future_hour + 1]
            this_hour_24_int = int(datetime.datetime.fromtimestamp(
                this_hour.time).strftime("%H"))
            if this_hour_24_int <= 11:
                ampm = 'a.m.'
            else:
                ampm = 'p.m.'
            this_hour_12_int = int(datetime.datetime.fromtimestamp(
                this_hour.time).strftime("%I"))
            this_hour_string = "{} {}".format(str(this_hour_12_int), ampm)
            multiplier += 2
            self.weather_common.display_subwindow(
                this_hour, this_hour_string, multiplier)

        # Update the display
        pygame.display.update()




import struct
import time

# Header Byte Types
addr = 0x04
func = 0x06
fullBright = 0x7e;

funccode = {
    "All Off": 0x0,
    "All On": 0x1,
    "On": 0x2,
    "Off": 0x3,
    "Dim": 0x4,
    "Bright": 0x5,
    "All Off": 0x6,
    "ExtCode": 0x7,
    "HailReq": 0x8,
    "HailAck": 0x9,
    "Preset1": 0xA,
    "Preset2": 0xB,
    "ExtDataXfer": 0xC,
    "StatusOn": 0xD,
    "StatusOff": 0xE,
    "StatusReq": 0xF
    }


housecode = {
    "A": 0x6,
    "B": 0xE,
    "C": 0x2,
    "D": 0xA,
    "E": 0x1,
    "F": 0x9,
    "G": 0x5,
    "H": 0xD,
    "I": 0x7,
    "J": 0xF,
    "K": 0x3,
    "L": 0xB,
    "M": 0x0,
    "N": 0x8,
    "O": 0x4,
    "P": 0xC
    }

unitcode = {
    "1": 0x6,
    "2": 0xE,
    "3": 0x2,
    "4": 0xA,
    "5": 0x1,
    "6": 0x9,
    "7": 0x5,
    "8": 0xD,
    "9": 0x7,
    "10": 0xF,
    "11": 0x3,
    "12": 0xB,
    "13": 0x0,
    "14": 0x8,
    "15": 0x4,
    "16": 0xC
    }

# Given a byte string, calculate the checksum by adding all the
# bytes and then returning the bitwise AND with 0xFF.
#==============================================================
def X10_Checksum( s ):
    sum = 0
    for c in s:
        sum = sum + ord(c)
    return sum & 0xFF


# Send a string to the X10 CM11a module.  The module replies with
# a checksum.  On a good checksum, we send an ACK.  After the ACK,
# the module replies with a READY.  Note, the ACK value is zero and
# the READY value is 0x55.  Also note, it make take up to two
# seconds for the module to respond on some commands.
#==============================================================
def X10_Send( ser, s ):
    ret = True  # Init return code.

    ser.flushInput()
    ser.write( s )
    c = ser.read( 1 )
    cs = X10_Checksum( s )  # Compute checksum of the bytes.

    if (len(c) == 1) and (ord(c) == cs):  # Good Checksum
        ser.write( chr(0x00) )  # Send ACK
        c = ser.read(1)  # Look for X10 ready.
        if (len(c) == 1) and (ord(c) == 0x55):
            ret = True  # All Good
        else:
            print("Err: Missing X10 ready response.")
            ret = False
    else:
        print(f"Checksum Err / Len = {len(c)}")
        if len(c) == 1:
            print(f"Checksum -> {ord(c)} expecting {cs}")
        ret = False

    return ret  # Return True on Good & False on Bad



#==============================================================
def X10_On( ser, h, u ):

    ser.flushInput()
    b = struct.pack( 'BB', addr, (h << 4 ) | (u & 0x0F) )
    if not X10_Send( ser, b ):
        print("X10 Error send first ON string.")
        return False
    b = struct.pack( 'BB', func, (h << 4 ) | funccode['On'] )
    if not X10_Send( ser, b ):
        print("X10 Error send second ON string.")
        return False

    return True  # Everything must be OK.

#==============================================================
def X10_Off( ser, h, u ):

    ser.flushInput()
    b = struct.pack( 'BB', addr, (h << 4 ) | (u & 0x0F) )
    if not X10_Send( ser, b ):
        print("X10 Error send first OFF string.")
        return False
    b = struct.pack( 'BB', func, (h << 4 ) | funccode['Off'] )
    if not X10_Send( ser, b ):
        print("X10 Error send second OFF string.")
        return False

    return True  # Everything must be OK.

#==============================================================
def X10_Bright( ser, h, u ):
    ret = True
    to = ser.timeout  # Save timeout
    ser.timeout = 5
    ser.flushInput()
    b = struct.pack( 'BB', addr, (h << 4 ) | (u & 0x0F) )
    if not X10_Send( ser, b ):
        print("X10 Error send first Bright string.")
        ret = False
    if ret == True:
        b = struct.pack( 'BB', fullBright, (h << 4 ) | funccode['Bright'] )
        if not X10_Send( ser, b ):
            print("X10 Error send second Bright string.")
            ret = False

    ser.timeout = to  # Restore timeout to orginal value.
    return ret  # True = OK / False = Error.



# Ask the CM11A for its status.  In short order, the CM11A should
# respond with 14 bytes of status.
#==============================================================
def X10_Status( ser ):
    to = ser.timeout  # Save current timeout.
    ser.timeout = 0.1  # Status bytes should be quick.
    ret = False

    ser.flushInput()
    time.sleep( 0.1 )
    ser.write( chr(0x8b) )
    c = ser.read( 14 )  # The module should return 14 bytes of info.
    if len(c) >= 13:
        i = 0
        for a in c: 
            print(f"{i} : {hex(ord(a))}")
            i = i + 1
        print("X10 status OK.")
        ret = True
    else:
        print("X10 status is BAD.")
        print(f"X10 string len: {len(c)}")
        i = 0
        for a in c: 
            print(f"{i} : {hex(ord(a))}")
            i = i + 1
        ret = False

    ser.write( chr(0x00) )  # Send an ACK.
    ser.timeout = to  # Restore timeout value.
    return (ret, c)


# The CM11A X10 will NOT do anything until its clock is set.
# The following code just sets the clock to something so the 
# interface can be used.
#==============================================================
def X10_SetClock( ser ):
    # Since I don't use the clock in the CM11A this code just
    # sets the clock to some junk so the module can start working.
    s = b"\x9b\x32\x66\x07\xf4\x04\x60"

    ser.flushInput()
    time.sleep( 0.5 )  # Wait a bit after getting a 0xA5.
    ser.flushInput()
    ser.write( s )
    print("Resetting X10 clock.")
    c = ser.read(1)  # Readback checksum.
    cs = X10_Checksum( s[1:] )
    if (len(c) == 1) and (ord(c) == cs):
        ser.write( chr(0x00) )
        c = ser.read( 1 )
        if (len(c) == 1) and (ord(c) == 0x55):
            print("X10 Clock set.")
        else:
            print("X10 final 0x55 marker missing.")
    else:
        ser.write( chr(0x00) )
        print("Bad checksum from X10 interface.")
        print(f"X10 returned: {hex(ord(c))}")
        print(f"Expected Checksum: {hex(cs)}")





"""Module setup."""

import runpy
from setuptools import setup, find_namespace_packages, find_packages

PACKAGE_NAME = "piweatherrock"
version_meta = runpy.run_path("./version.py")
VERSION = version_meta["__version__"]


with open("README.md", "r") as fh:
    long_description = fh.read()


def parse_requirements(filename):
    """Load requirements from a pip requirements file."""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


if __name__ == "__main__":
    setup(
        name=PACKAGE_NAME,
        author="Gene Liverman",
        author_email="gene@technicalissues.us",
        version=VERSION,
        packages=find_packages(),
        include_package_data=True,
        install_requires=parse_requirements("requirements.txt"),
        python_requires=">=3.6",
        scripts=[
            'scripts/pwr-ui',
            'scripts/pwr-config-upgrade',
        ],
        description="Provides forecast data from ClimaCell for PiWeatherRock",
        long_description=long_description,
        long_description_content_type="text/markdown",
        license='MIT',
        url='https://github.com/genebean/PiWeatherRock',
        classifiers=[
            'License :: OSI Approved :: MIT License',
        ],
    )



# -*- coding: utf-8 -*-
# Copyright (c) 2014 Jim Kemp <kemp.jim@gmail.com>
# Copyright (c) 2017 Gene Liverman <gene@technicalissues.us>
# Distributed under the MIT License (https://opensource.org/licenses/MIT)

import datetime
import pygame
import time


class PluginInfo:
    """
    Displays a screen providing information about this application
    along with the time of sunrise and sunset. The time and date are
    displayed in a different place than on the daily and hourly
    screens and there is no border. This is a conscious descison to
    help prevent screen burn-in.
    """

    def __init__(self, weather_rock):
        self.screen = None
        self.weather = None
        self.last_update_check = None
        self.xmax = None
        self.ymax = None
        self.time_date_small_text_height = None
        self.time_date_text_height = None
        self.time_date_y_position = None
        self.time_date_small_y_position = None
        self.sunrise_string = None
        self.sunset_string = None

        self.get_rock_values(weather_rock)

    def get_rock_values(self, weather_rock):
        self.screen = weather_rock.screen
        self.weather = weather_rock.weather
        self.last_update_check = weather_rock.last_update_check
        self.xmax = weather_rock.xmax
        self.ymax = weather_rock.ymax
        self.time_date_small_text_height = weather_rock.time_date_small_text_height
        self.time_date_text_height = weather_rock.time_date_text_height
        self.time_date_y_position = weather_rock.time_date_y_position
        self.time_date_small_y_position = weather_rock.time_date_small_y_position
        self.sunrise_string = weather_rock.sunrise_string
        self.sunset_string = weather_rock.sunset_string

    def disp_info(self, weather_rock):
        self.get_rock_values(weather_rock)

        (in_daylight, day_hrs, day_mins, seconds_til_daylight,
         delta_seconds_til_dark) = self.daylight(self.weather)

        # Fill the screen with black
        self.screen.fill((0, 0, 0))
        xmin = 10
        lines = 5
        line_color = (0, 0, 0)
        text_color = (255, 255, 255)
        font_name = "freesans"

        # Draw Screen Border
        pygame.draw.line(self.screen, line_color,
                         (xmin, 0), (self.xmax, 0), lines)
        pygame.draw.line(self.screen, line_color,
                         (xmin, 0), (xmin, self.ymax), lines)
        pygame.draw.line(self.screen, line_color,
                         (xmin, self.ymax), (self.xmax, self.ymax), lines)
        pygame.draw.line(self.screen, line_color,
                         (self.xmax, 0), (self.xmax, self.ymax), lines)
        pygame.draw.line(self.screen, line_color,
                         (xmin, self.ymax * 0.15),
                         (self.xmax, self.ymax * 0.15), lines)

        time_height_large = self.time_date_text_height
        time_height_small = self.time_date_small_text_height

        # Time & Date
        regular_font = pygame.font.SysFont(
            font_name, int(self.ymax * time_height_large), bold=1)
        small_font = pygame.font.SysFont(
            font_name, int(self.ymax * time_height_small), bold=1)

        hours_and_minutes = time.strftime("%I:%M", time.localtime())
        am_pm = time.strftime(" %p", time.localtime())

        rendered_hours_and_minutes = regular_font.render(
            hours_and_minutes, True, text_color)
        (tx1, ty1) = rendered_hours_and_minutes.get_size()
        rendered_am_pm = small_font.render(am_pm, True, text_color)
        (tx2, ty2) = rendered_am_pm.get_size()

        tp = self.xmax / 2 - (tx1 + tx2) / 2
        self.screen.blit(rendered_hours_and_minutes,
                         (tp, self.time_date_y_position))
        self.screen.blit(rendered_am_pm,
                         (tp + tx1 + 3, self.time_date_small_y_position))

        self.string_print(
            "A weather rock powered by Dark Sky", small_font,
            self.xmax * 0.05, 3, text_color)

        self.string_print(
            "Sunrise: %s" % self.sunrise_string,
            small_font, self.xmax * 0.05, 4, text_color)

        self.string_print(
            "Sunset:  %s" % self.sunset_string,
            small_font, self.xmax * 0.05, 5, text_color)

        text = "Daylight: %d hrs %02d min" % (day_hrs, day_mins)
        self.string_print(text, small_font, self.xmax * 0.05, 6, text_color)

        # leaving row 7 blank

        if in_daylight:
            text = "Sunset in %d hrs %02d min" % self.stot(
                delta_seconds_til_dark)
        else:
            text = "Sunrise in %d hrs %02d min" % self.stot(
                seconds_til_daylight)
        self.string_print(text, small_font, self.xmax * 0.05, 8, text_color)

        # leaving row 9 blank

        text = "Weather checked at"
        self.string_print(text, small_font, self.xmax * 0.05, 10, text_color)

        text = "    %s" % time.strftime(
            "%I:%M:%S %p %Z on %a. %d %b %Y ",
            time.localtime(self.last_update_check))
        self.string_print(text, small_font, self.xmax * 0.05, 11, text_color)

        # Update the display
        pygame.display.update()

    def string_print(self, text, font, x, line_number, text_color):
        """
        Prints a line of text on the display
        """
        rendered_font = font.render(text, True, text_color)
        self.screen.blit(rendered_font, (x, self.ymax * 0.075 * line_number))

    def daylight(self, weather):
        """
        Given a weather forecast containing sunrise and sunset unix
        timestamps, return true if current local time is between sunrise
        and sunset. In other words, return true if it's daytime and the sun
        is up. Also, return the number of hours:minutes of daylight in this
        day. Lastly, return the number of seconds until daybreak and sunset.
        If it's dark, daybreak is set to the number of seconds until sunrise.
        If it daytime, sunset is set to the number of seconds until the sun
        sets.

        So, five things are returned as:
         (in_daylight,
          day_hrs,
          day_mins,
          seconds_til_daylight,
          delta_seconds_til_dark).
        """
        in_daylight = False    # Default return code.

        # Get current datetime with tz's local day and time.
        tNow = datetime.datetime.now()

        # Build a datetime variable from a unix timestamp for today's sunrise.
        tSunrise = datetime.datetime.fromtimestamp(
            weather.daily[0].sunriseTime)
        tSunset = datetime.datetime.fromtimestamp(
            weather.daily[0].sunsetTime)

        # Test if current time is between sunrise and sunset.
        if (tNow > tSunrise) and (tNow < tSunset):
            in_daylight = True        # We're in Daytime
            delta_seconds_til_dark = tSunset - tNow
            seconds_til_daylight = 0
        else:
            in_daylight = False        # We're in Nighttime
            delta_seconds_til_dark = 0            # Seconds until dark.
            # Delta seconds until daybreak.
            if tNow > tSunset:
                # Must be evening - compute sunrise as time left today
                # plus time from midnight tomorrow.
                sunrise_tomorrow = datetime.datetime.fromtimestamp(
                    weather.daily[1].sunriseTime)
                seconds_til_daylight = sunrise_tomorrow - tNow
            else:
                # Else, must be early morning hours. Time to sunrise is
                # just the delta between sunrise and now.
                seconds_til_daylight = tSunrise - tNow

        # Compute the delta time (in seconds) between sunrise and set.
        dDaySec = tSunset - tSunrise           # timedelta in seconds
        (day_hrs, dayMin) = self.stot(dDaySec)  # split into hours and minutes.

        return (in_daylight, day_hrs, dayMin, seconds_til_daylight,
                delta_seconds_til_dark)

    # Helper function to which takes seconds and returns (hours, minutes).
    # ###########################################################################
    def stot(self, sec):
        mins = sec.seconds // 60
        hrs = mins // 60
        return (hrs, mins % 60)
